@@ -1,6 +1,6 @@
 use std::mem::MaybeUninit;
 
-use util::math::{Fixed, FloatOutlineIterExt, I16Dot16, OutlineEvent, Point2, Point2f};
+use util::math::{Fixed, FloatOutlineIterExt, I16Dot16, OutlineEvent, Point2, Point2f, Vec2};
 
 mod tile;
 
@@ -48,6 +48,10 @@ fn to_tile_fixed(v: I16Dot16) -> U2Dot14 {
     U2Dot14::from_raw((v.into_raw() >> 2).clamp(0, i32::from(U2Dot14::MAX.into_raw())) as u16)
 }
 
+fn from_tile_fixed(v: U2Dot14) -> I16Dot16 {
+    I16Dot16::from_raw(i32::from(v.into_raw()) << 2)
+}
+
 impl StripRasterizer {
     pub fn new() -> Self {
         Self {
@@ -158,9 +162,9 @@ impl StripRasterizer {
         end_tile_x: i16,
         tile_y: u16,
         mut bottom_inner_x: I16Dot16,
-        bottom_inner_y: U2Dot14,
+        mut bottom_inner_y: U2Dot14,
         mut top_inner_x: I16Dot16,
-        top_inner_y: U2Dot14,
+        mut top_inner_y: U2Dot14,
         winding: Winding,
     ) {
         debug_assert!(bottom_inner_y <= top_inner_y);
@@ -171,41 +175,83 @@ impl StripRasterizer {
         } else {
             debug_assert!(top_inner_x <= 0);
             let o = I16Dot16::new(4 * i32::from(tile_x - end_tile_x));
-            bottom_inner_x = o + bottom_inner_x;
-            top_inner_x = o + top_inner_x;
+            bottom_inner_x += o;
+            top_inner_x += o;
             (end_tile_x, tile_x - end_tile_x + 1)
         };
 
-        if pos_x + width <= 0 {
+        let upos_x = if let Ok(upos_x) = u16::try_from(pos_x) {
+            upos_x
+        } else {
+            // If `pos_x < 0` then one or both of this line's points are to the left of
+            // the y-axis and hence our canvas. This means we need to clip it and insert
+            // a vertical line along the y-axis for the clipped part.
+            let (left_fill_bottom, left_fill_top) = if pos_x + width <= 0 {
+                // If `pos_x + width <= 0` then both of our points are left of the y-axis.
+                // This means we can just throw away the whole line.
+                let prev_top = top_inner_y;
+                top_inner_y = bottom_inner_y;
+                (bottom_inner_y, prev_top)
+            } else {
+                // Otherwise *only one* of the points is to the left of the y-axis.
+                // This is trickier since we need to actually split it into a vertical
+                // part and a remainder.
+                let dy = (from_tile_fixed(top_inner_y) - from_tile_fixed(bottom_inner_y))
+                    / (top_inner_x - bottom_inner_x);
+                let bottom_abs_x = bottom_inner_x + i32::from(pos_x) * 4;
+                let top_abs_x = top_inner_x + i32::from(pos_x) * 4;
+                debug_assert!((bottom_abs_x < 0) ^ (top_abs_x < 0));
+
+                if bottom_abs_x < 0 {
+                    bottom_inner_x = I16Dot16::ZERO;
+                    top_inner_x += i32::from(pos_x) * 4;
+
+                    let prev_y = bottom_inner_y;
+                    bottom_inner_y =
+                        to_tile_fixed(from_tile_fixed(bottom_inner_y) - (dy * bottom_abs_x));
+                    (prev_y, bottom_inner_y)
+                } else {
+                    top_inner_x = I16Dot16::ZERO;
+                    bottom_inner_x += i32::from(pos_x) * 4;
+
+                    let prev_y = top_inner_y;
+                    top_inner_y = to_tile_fixed(from_tile_fixed(top_inner_y) - (dy * top_abs_x));
+                    (top_inner_y, prev_y)
+                }
+            };
+
+            debug_assert!(left_fill_bottom <= left_fill_top);
+
             self.tiles.push(Tile {
                 pos: Point2::new(0, tile_y),
                 width: 1,
                 line: TileLine {
                     bottom_x: I16Dot16::ZERO,
-                    bottom_y: bottom_inner_y,
+                    bottom_y: left_fill_bottom,
                     top_x: I16Dot16::ZERO,
-                    top_y: top_inner_y,
+                    top_y: left_fill_top,
                 },
                 winding,
             });
-        } else {
-            if pos_x < 0 {
-                // TODO: Implement this case!
+
+            if bottom_inner_y >= top_inner_y {
                 return;
             }
 
-            self.tiles.push(Tile {
-                pos: Point2::new(pos_x as u16, tile_y),
-                width: width as u16,
-                line: TileLine {
-                    bottom_x: bottom_inner_x,
-                    bottom_y: bottom_inner_y,
-                    top_x: top_inner_x,
-                    top_y: top_inner_y,
-                },
-                winding,
-            });
-        }
+            0
+        };
+
+        self.tiles.push(Tile {
+            pos: Point2::new(upos_x, tile_y),
+            width: width as u16,
+            line: TileLine {
+                bottom_x: bottom_inner_x,
+                bottom_y: bottom_inner_y,
+                top_x: top_inner_x,
+                top_y: top_inner_y,
+            },
+            winding,
+        });
     }
 
     fn add_tiles(&mut self, mut bottom: Point2<I16Dot16>, top: Point2<I16Dot16>, winding: Winding) {
@@ -226,7 +272,9 @@ impl StripRasterizer {
         let end_inner_y16 = top.y - tile_to_coord(end_tile_y as i16);
         let end_inner_y = to_tile_fixed(end_inner_y16);
         let mut current_inner_x = bottom.x - floor_to_tile(bottom.x.floor());
-        let dx4 = dx * 4;
+        // TODO: Fixed::saturating_mul or something
+        //       would be nice to be generally more careful about overflows in this file
+        let dx4 = I16Dot16::from_raw(I16Dot16::into_raw(dx).saturating_mul(4));
 
         let bottom_inner_y = bottom.y - floor_to_tile(bottom.y);
         if tile_y == end_tile_y {
@@ -295,13 +343,13 @@ impl StripRasterizer {
     }
 }
 
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, Hash, PartialEq, Eq)]
 pub struct Strips {
     strips: Vec<Strip>,
     alpha_buffer: AlphaBuffer,
 }
 
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, Hash, PartialEq, Eq)]
 struct AlphaBuffer(Vec<u64>);
 
 impl AlphaBuffer {
@@ -327,7 +375,7 @@ impl AlphaBuffer {
     }
 }
 
-#[derive(Debug, Clone, Copy)]
+#[derive(Debug, Clone, Copy, Hash, PartialEq, Eq)]
 struct Strip {
     pos: Point2<u16>,
     width: u16,
@@ -343,39 +391,65 @@ impl Strips {
         }
     }
 
-    pub fn paint_to(&self, buffer: &mut [u8], width: usize, height: usize, stride: usize) {
-        self.blend_to(buffer, |out, value| *out = value, width, height, stride)
+    pub fn paint_to(&self, target: super::RenderTargetView<u8>) {
+        self.blend_to(target, |out, value| *out = value)
     }
 
     pub fn blend_to<P: Copy>(
         &self,
-        buffer: &mut [P],
-        mut blend_func: impl FnMut(&mut P, u8),
-        width: usize,
-        height: usize,
-        stride: usize,
+        target: super::RenderTargetView<P>,
+        blend_func: impl FnMut(&mut P, u8),
     ) {
-        assert!(buffer.len() >= height * stride);
-        assert!(stride >= width);
+        self.blend_to_at(target, blend_func, Vec2::new(0, 0));
+    }
 
+    pub fn blend_to_at<P: Copy>(
+        &self,
+        target: super::RenderTargetView<P>,
+        mut blend_func: impl FnMut(&mut P, u8),
+        offset: Vec2<i32>,
+    ) {
         for op in self.paint_iter() {
             let pos = op.pos();
-            let out_pos = Point2::new(usize::from(pos.x) * 4, usize::from(pos.y) * 4);
-            if out_pos.y >= height || out_pos.x >= width {
-                continue;
-            }
+            let op_width = op.width() as u32;
+            let (out_off, out_pos) = {
+                let out_sx = pos.x as i32 * 4 + offset.x;
+                let out_sy = pos.y as i32 * 4 + offset.y;
 
-            let op_width = op.width();
-            let out_width = op_width.min(width - out_pos.x);
-            let out_height = 4.min(height - out_pos.y);
-            let mut current_out =
-                unsafe { buffer.as_mut_ptr().add(out_pos.y * stride + out_pos.x) };
-            let row_step = stride - out_width;
+                if out_sx <= -(op_width as i32) || out_sy <= -4 {
+                    continue;
+                }
+
+                let out_offx = -out_sx.min(0) as u32;
+                let out_offy = -out_sy.min(0) as u32;
+                let out_x = out_sx.max(0) as u32;
+                let out_y = out_sy.max(0) as u32;
+
+                if out_x >= target.width || out_y >= target.height {
+                    continue;
+                }
+
+                (Vec2::new(out_offx, out_offy), Point2::new(out_x, out_y))
+            };
+
+            let out_width = (op_width - out_off.x).min(target.width - out_pos.x);
+            let out_height = (4 - out_off.y).min(target.height - out_pos.y);
+            let mut current_out = unsafe {
+                target
+                    .buffer
+                    .as_mut_ptr()
+                    .add(out_pos.y as usize * target.stride as usize + out_pos.x as usize)
+            };
+            let row_step = (target.stride - out_width) as usize;
 
             match op {
                 StripPaintOp::Copy(op) => {
-                    let mut current_src = op.buffer.as_ptr();
-                    let src_row_step = op_width - out_width;
+                    let mut current_src = unsafe {
+                        op.buffer
+                            .as_ptr()
+                            .add(out_off.y as usize * op_width as usize + out_off.x as usize)
+                    };
+                    let src_row_step = (op_width - out_width) as usize;
 
                     for _ in 0..out_height {
                         unsafe {
@@ -391,7 +465,7 @@ impl Strips {
                     }
                 }
                 StripPaintOp::Fill(op) => {
-                    for &alpha in &op.alpha[..out_height] {
+                    for &alpha in &op.alpha[out_off.y as usize..out_height as usize] {
                         unsafe {
                             for _ in 0..out_width {
                                 blend_func(&mut *current_out, alpha);
@@ -417,12 +491,12 @@ pub struct StripPaintIter<'a> {
 #[derive(Debug, Clone, Copy)]
 pub enum StripPaintOp<'a> {
     Copy(StripCopyOp<'a>),
-    Fill(StripFillOp),
+    Fill(StripFillOp<'a>),
 }
 
 impl StripPaintOp<'_> {
     #[inline]
-    fn pos(&self) -> Point2<u16> {
+    pub fn pos(&self) -> Point2<u16> {
         match self {
             Self::Copy(x) => x.pos,
             Self::Fill(x) => x.pos,
@@ -430,7 +504,7 @@ impl StripPaintOp<'_> {
     }
 
     #[inline]
-    fn width(&self) -> usize {
+    pub fn width(&self) -> usize {
         match self {
             StripPaintOp::Copy(x) => x.width().into(),
             StripPaintOp::Fill(x) => usize::from(x.width) * 4,
@@ -444,7 +518,7 @@ pub struct StripCopyOp<'a> {
     pub buffer: &'a [u8],
 }
 
-impl StripCopyOp<'_> {
+impl<'a> StripCopyOp<'a> {
     #[inline]
     pub fn height(&self) -> u16 {
         4
@@ -454,13 +528,33 @@ impl StripCopyOp<'_> {
     pub fn width(&self) -> u16 {
         (self.buffer.len() / 4) as u16
     }
+
+    #[inline]
+    pub fn to_texture(self) -> super::Texture<'a> {
+        super::Texture {
+            width: u32::from(self.width()),
+            height: u32::from(self.height()),
+            data: super::TextureData::BorrowedMono(self.buffer),
+        }
+    }
 }
 
 #[derive(Debug, Clone, Copy)]
-pub struct StripFillOp {
+pub struct StripFillOp<'a> {
     pub pos: Point2<u16>,
     pub width: u16,
-    pub alpha: [u8; 4],
+    pub alpha: &'a [u8; 4],
+}
+
+impl<'a> StripFillOp<'a> {
+    #[inline]
+    pub fn to_vertical_texture(self) -> super::Texture<'a> {
+        super::Texture {
+            width: 1,
+            height: 4,
+            data: super::TextureData::BorrowedMono(self.alpha),
+        }
+    }
 }
 
 impl<'a> Iterator for StripPaintIter<'a> {
@@ -475,7 +569,7 @@ impl<'a> Iterator for StripPaintIter<'a> {
             return Some(StripPaintOp::Fill(StripFillOp {
                 pos: Point2::new(last_x, next.pos.y),
                 width: next.pos.x - last_x,
-                alpha: next.fill_previous,
+                alpha: &next.fill_previous,
             }));
         }
 
@@ -562,7 +656,7 @@ mod test {
         math::{Outline, StaticOutline, Vec2},
     };
 
-    use crate::sw::StripRasterizer;
+    use crate::sw::{RenderTargetView, StripRasterizer};
 
     pub fn compare(size: Vec2<u32>, coverage: &[u8], expected: &[u8]) {
         let mut matches = true;
@@ -641,12 +735,7 @@ mod test {
 
         let strips = rasterizer.rasterize();
         let mut coverage = vec![0; size.x as usize * size.y as usize];
-        strips.paint_to(
-            &mut coverage,
-            size.x as usize,
-            size.y as usize,
-            size.x as usize,
-        );
+        strips.paint_to(RenderTargetView::new(&mut coverage, size.x, size.y, size.x));
 
         compare(size, &coverage, expected);
     }
@@ -1011,4 +1100,72 @@ mod test {
             EXPECTED,
         );
     }
+
+    #[test]
+    fn negative_xy() {
+        #[rustfmt::skip]
+        const EXPECTED: &[u8] = &[
+            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+            0x7F, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+            0xFF, 0x7F, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+            0xFF, 0xFF, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00,
+            0xFF, 0xFF, 0xFF, 0x80, 0x00, 0x00, 0x00, 0x00,
+        ];
+
+        test_outline(
+            &make_static_outline![
+                #move_to (-4.0, -4.0);
+                line_to (8.0, -4.0);
+                line_to (-4.0, 8.0);
+            ],
+            EXPECTED,
+        );
+    }
+
+    #[test]
+    fn negative_xy_inv() {
+        #[rustfmt::skip]
+        const EXPECTED: &[u8] = &[
+            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+            0x7F, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+            0xFF, 0x7F, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+            0xFF, 0xFF, 0x7F, 0x00, 0x00, 0x00, 0x00, 0x00,
+            0xFF, 0xFF, 0xFF, 0x80, 0x00, 0x00, 0x00, 0x00,
+        ];
+
+        test_outline(
+            &make_static_outline![
+                #move_to (-4.0, 8.0);
+                line_to (8.0, -4.0);
+                line_to (-4.0, -4.0);
+            ],
+            EXPECTED,
+        );
+    }
+
+    #[test]
+    fn negative_xy_mirror() {
+        #[rustfmt::skip]
+        const EXPECTED: &[u8] = &[
+            0x00, 0x00, 0x15, 0xAA,
+            0x00, 0x55, 0xEA, 0xFF,
+            0xAA, 0xFF, 0xFF, 0xFF,
+            0xFF, 0xFF, 0xFF, 0xFF,
+        ];
+
+        test_outline(
+            &make_static_outline![
+                #move_to (4.0, -4.0);
+                line_to (-8.0, -4.0);
+                line_to (4.0, 4.0);
+            ],
+            EXPECTED,
+        );
+    }
 }
