@@ -763,14 +763,19 @@ impl FontImpl for Font {
             };
 
             let texture = rasterizer.create_packed_texture_mapped(
-                scaled_width,
-                scaled_height,
+                Vec2::new(scaled_width, scaled_height),
                 if matches!(pixel_mode, CopyPixelMode::Bgra32) {
                     PixelFormat::Bgra
                 } else {
                     PixelFormat::Mono
                 },
-                Box::new(|buffer_data, stride| {
+                Box::new(|mut target| {
+                    // TODO: The software rasterizer now supports texture scaling.
+                    //       However that cannot really be used by this code because it expects
+                    //       sane textures with a positive stride (and FreeType's can be negative).
+                    //       I think I will defer switching to that here until we start using
+                    //       skrifa for parsing fonts.
+                    let stride = target.stride() as usize;
                     macro_rules! copy_font_bitmap_with {
                         ($pixel_mode: expr) => {
                             copy_font_bitmap::<$pixel_mode>(
@@ -778,7 +783,7 @@ impl FontImpl for Font {
                                 bitmap.pitch as isize,
                                 bitmap.width as u32,
                                 bitmap.rows as u32,
-                                buffer_data,
+                                target.buffer_mut(),
                                 stride,
                                 scale,
                                 scaled_width,
