@@ -8,7 +8,6 @@ use text_sys::*;
 use util::{
     make_static_outline,
     math::{I16Dot16, I26Dot6, Outline, OutlineIterExt as _, Point2, Rect2, StaticOutline, Vec2},
-    slice_assume_init_mut,
 };
 
 use super::{FaceImpl, FontImpl, FontMetrics, GlyphMetrics, SingleGlyphBitmap};
@@ -457,19 +456,14 @@ impl FontImpl for Font {
         } = self.draw_glyph(index, offset, &mut StripRasterizer::new());
         let texture = unsafe {
             rasterizer.create_packed_texture_mapped(
-                texture_size.x,
-                texture_size.y,
+                texture_size,
                 PixelFormat::Mono,
-                Box::new(|texture, stride| {
-                    texture.fill(MaybeUninit::new(0));
-                    let texture = slice_assume_init_mut(texture);
+                Box::new(|mut target| {
+                    target.buffer_mut().fill(MaybeUninit::new(0));
 
-                    strips.paint_to(
-                        texture,
-                        texture_size.x as usize,
-                        texture_size.y as usize,
-                        stride,
-                    );
+                    strips.blend_to(target, |out, value| {
+                        out.write(value);
+                    });
                 }),
             )
         };
